/*
 * Test.java
 * Created on 22.04.2003
 */

package search;

import java.text.DecimalFormat;

import search.graph.Examples;
import search.graph.Graph;
import search.graph.Node;
import search.graph.Path;
import search.graph.SearchStrategy;
import search.graph.Solver;
import search.util.TreeAlgorithms;

/**
 * Einfache Testklasse, die ohne GUI auskommt.
 * Als Ergebnis werden der gefundene Weg und der Suchbaum ausgegeben.
 * 
 * @author Erich Ehses
 */
public final class Test {
    
    private static DecimalFormat f = new DecimalFormat("0.0");
    
    private static final SearchStrategy[] strategies =
        {
            SearchStrategy.LIFO,
            SearchStrategy.FIFO,
            SearchStrategy.PATH_LENGTH,
            SearchStrategy.HILL_CLIMBING,
            SearchStrategy.A_STAR
        };
        
    /**
     * Main-Methode zum Test der Graphsuche.
     * Die Kommandozeile enthlt:
     * <ul>
     * <li>args[0]: Nummer des Algorithmus (0 - 4).</li>
     * <li>args[1]: Name des Startortes.</li>
     * <li>args[2]: Name des Zielortes.</li>
     * </ul>
     * @param args Kommandozeile. 
     */
    public static void main(String[] args) {
        SearchStrategy strategy = strategies[Integer.parseInt(args[0])];
        String startName = args[1];
        String goalName = args[2];
        
        Graph g = Examples.germany();
        Node start = g.findNode(startName);
        Node ziel = g.findNode(goalName);
        
        Path p = Solver.solve(strategy, start, ziel);
            
        System.out.print("Der Weg von "+start+" nach "+ziel);
        System.out.println(" betrgt "+f.format(p.getPathLength())+" km.");
        
        System.out.println("\nEr verluft ber:");
        Object[] nodes = p.getNodes().toArray();
        for (int i = nodes.length-1; i>=0; i--) {
            System.out.println("    "+nodes[i]);
        }
        System.out.println();

        System.out.println("\nEs gibt : " + (TreeAlgorithms.numberOfInnerNodes(p.getRoot())-1) + " innere Knoten");
        //System.out.println("\nDer Suchbaum lautet: ");
        //TreeAlgorithms.printTree(p.getRoot(), 0);
    }
    
}
