/*
 * Edge.java
 * Created on 21.04.2003
 */
package search.graph;

import java.awt.*;

/**
 * Klasse zum Speichern einer Graphkante.
 * @author Erich Ehses.
 */
public abstract class Edge implements Teilstrecke {
    protected Node l1;            // Anfangsknoten
    protected Node l2;            // Endknoten
    protected double edgeLength;  // Kantenlaenge
    protected Color color;

    public Color getColor(){
        return color;
    }

    /**
     * Konstruktor. Die Kantenlaenge ist gleich der Luftlinenentfernung
     * der Endknoten.
     * @param l1 Anfangsknoten.
     * @param l2 Endknoten.
     */
    public Edge(Node l1, Node l2) {
        this(l1, l2, l1.distTo(l2));
    }
    
    /**
     * Konstruktor. Die Kantenlaenge wird explizit angegeben.
     * @param l1 Anfangsknoten.
     * @param l2 Endknoten.
     * @param length Kantenlaenge.
     */
    public Edge(Node l1, Node l2, double length) {
        this.l1 = l1;
        this.l2 = l2;
        this.edgeLength = length;
    }
    
    /**
     * Gibt einen Endknoten zurueck.
     * @param nr Nummer des Knotens (0 oder 1).
     * @return Knotenobjekt.
     * @throws IllegalArgumentException wenn <code>nr</code> nicht 0 oder 1.
     */ 
    public Node getNode(int nr) {
        if (nr < 0 || nr > 1) throw new IllegalArgumentException();
        return nr == 0? l1: l2;
    }

    /**
     * Bestimmt den Partner zu einem der Endknoten.
     * @param origin gegebener Endknoten.
     * @return gesuchter Endknoten.
     */
    public Node getPartner(Node origin) {
        return (origin == l1)? l2: l1;
    }
    
    /**
     * Bestimmt die Laenge der Kante.
     * @return Laenge der Kante.
     */
    public double getEdgeLength() {
        return edgeLength;
    }

    public boolean istBefahrbarDurchKFZ() {
        return true;
    }
}
