/*
 * Graph.java
 * Created on 21.04.2003
 */
package search.graph;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;

/**
 * Speichert einen kompletten (geometrischen) Graphen.
 * @author Erich Ehses.
 */
public final class Graph {
    private HashMap nodes = new HashMap();
    private ArrayList edges = new ArrayList();
    
    /**
     * Fuegt einen Knoten hinzu. Der Knotenname sollte eindeutig sein.
     * @param name Knotenname.
     * @param x x-Koordinate des Knotens.
     * @param y y-Koordinate des Knotens.
     */
    public void addNode(String name, double x, double y) {
        nodes.put(name, new Node(name, x, y));
    }
    
    /**
     * Fuegt eine neue Kante hinzu.
     * Die Laenge der Kante ist gleich der Entfernung der beiden Knoten.
     * Die beiden Endknoten muessen bereits existieren.
     * @param from Name des ersten Entknotens.
     * @param to Name des zweiten Endknotens.
     * @throws NullPointerException wenn einer der Endknoten nicht existiert.
     */
    public void addEdge(String from, String to, int typ) {
        Node fromNode = (Node)nodes.get(from);
        Node toNode = (Node)nodes.get(to);
        Teilstrecke newEdge=null;
        switch(typ){
            default: newEdge = new Verbindung(fromNode, toNode);
                break;

        }

        edges.add(newEdge);
        fromNode.addEdge(newEdge);
        toNode.addEdge(newEdge);
    }

    public void removeNode (Node node){
        System.out.println("***** Remove: "+node.getName());
        Iterator it = node.neighbours();
        while (it.hasNext()){
            Node temp = (Node) it.next();
            //2x
            Teilstrecke tempEdge = node.edgeTo(temp);
            edges.remove(tempEdge);
            temp.removeEdge(tempEdge);

            tempEdge = temp.edgeTo(node);
            edges.remove(tempEdge);
            node.removeEdge(tempEdge);

            temp.removeEntries(node);
            nodes.remove(node);
            node.isDead = true;
        }
    }

    /**
     * Findet des Knotenobjekt mit gegebenem Namen.
     * @param name Knotenname.
     * @return Knotenobjekt oder <code>null</code> wenn nicht gefunden.
     */
    public Node findNode(String name) {
        return (Node)nodes.get(name);
    }
    
    /**
     * Gibt ein Feld mit den sortierten Knotennamen zurueck.
     * @return Feld mit Knotennamen.
     */
    public String[] getNodeNames() {
        String[] names = new String[nodes.size()];
        Iterator iter = nodes.keySet().iterator();
        for (int i=0; i<names.length; i++) {
            names[i] = (String)iter.next();
        }
        Arrays.sort(names, Collator.getInstance(Locale.GERMANY));
        return names;
    }
    
    /**
     * Erzeugt eine Iterator ueber alle Knoten.
     * Die Reihenfolge der Knoten ist unbestimmt.
     * @return Iterator ueber alle Knoten.
     */
    public Iterator nodeIterator() {
        return nodes.values().iterator();
    }
    
    /**
     * Erzeugt einen Iterator ueber alle Kanten.
     * @return Iterator ueber alle Kanten.
     */
    public Iterator edgeIterator() {
        return edges.iterator();
    }
}
