/*
 * Node.java
 * Created on 21.04.2003
 */
package search.graph;

import search.util.RoutingTable;
import search.util.RoutingTableEntry;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

/**
 * Klasse zum Speichern eines Graphknotens mit geometrischer Information (2D).
 * @author Erich Ehses.
 */
public final class Node {
    
    private String name;         // Knotenname
    private double x;           // x-Koordinate
    private double y;           // y-Koordinate
    private Vector edges = new Vector();    // Kantenliste.
    private RoutingTable routingTable;
    public boolean isDead;

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    /**
     * Konstruktor.
     * @param name Knotenname.
     * @param x x-Koordinate.
     * @param y y-Koordinate.
     */
    public Node(String name, double x, double y) {
        this.name = name;
        this.x = x;
        this.y = y;
        routingTable = new RoutingTable();
    }

    public Node getBestNeighbourNode (Node goalNode){
        return routingTable.getBestNeighbourNode(goalNode);
    }

    public Node getNeighbourNodeWithShortestPath (Node goalNode){
        // falls es keinen Eintrag fr dieses Ziel gibt ...
        if (null!= routingTable.getEntryWithShortestPath(goalNode)){
            return routingTable.getEntryWithShortestPath(goalNode).getNeighbourNode();
        }
        // ... wird ein zuflliger Nachbarknoten gewhlt
        else {
            return getRandomNeighbourNode();
        }
    }

    public int getPheromoneValue (Node neighbourNode, Node goalNode){
        return routingTable.getPheromoneValue(neighbourNode, goalNode);
    }

    public void removeEntries (Node neighbourNode){
        routingTable.removeEntries(neighbourNode);
    }

    public Node getRandomNeighbourNode (){
        int edgecount = this.edges.size();
        int randomEdge = (int)((Math.random())*edgecount);
        return ((Edge)this.edges.get(randomEdge)).getPartner(this);
    }

    private Vector getProbabilityList (){
        return routingTable.getProbabilityList();
    }

    public void increasePheromoneValue (Node goalNode, Node lastVisitedNeighbourNode, Path path){
        /*String bestNeighbourName="";
        // alter bestNeighbourNode
        if (null != getBestNeighbourNode(goalNode)){
            bestNeighbourName = getBestNeighbourNode(goalNode).getName();
        }

        if (!goalNode.equals(lastVisitedNeighbourNode)){
            routingTable.increasePheromoneValue(goalNode, lastVisitedNeighbourNode, path);
            //System.out.println("(backward Ant) Pheromonwert von " + lastVisitedNeighbourNode.getName()+" -> "+goalNode.getName()+" in [" + this.getName() +"] erhht");
        }
        if (null != getBestNeighbourNode(goalNode)){
            // der bestNeighbourNode hat sich verndert
            if (!bestNeighbourName.equals(getBestNeighbourNode(goalNode).getName()) && !bestNeighbourName.equals("")){
                System.out.println("********* neuer, besserer oder quivalenter Weg nach "+goalNode.getName()+"  gefunden :");
                //System.out.println("");
                System.out.println("     Tabelle fr ["+this.getName()+"]");
                Vector probabilityList = getProbabilityList();
                for (int i=0; i<probabilityList.size(); i++){
                    RoutingTableEntry entry = (RoutingTableEntry)probabilityList.get(i);
                    System.out.println("     "+entry.getNeighbourNode()+" als Weg nach "+entry.getGoalNode()+" Gewichtung "+entry.getPheromoneValue() + " Weglnge : "+entry.getPathlength());
                }
                //System.out.println("");
            }
        }*/
        Node oldBest=null;
        Node newBest=null;
        double oldDistance=0.0;
        double newDistance=0.0;
        if (null != routingTable.getEntryWithShortestPath(goalNode)){
            oldBest = routingTable.getEntryWithShortestPath(goalNode).getNeighbourNode();
            oldDistance = routingTable.getEntryWithShortestPath(goalNode).getPathlength();
        }
        //if (!lastVisitedNeighbourNode.equals(goalNode)){
            routingTable.increasePheromoneValue(goalNode, lastVisitedNeighbourNode, path);
        //}
        if (null != routingTable.getEntryWithShortestPath(goalNode)){
            newBest = routingTable.getEntryWithShortestPath(goalNode).getNeighbourNode();
            newDistance = routingTable.getEntryWithShortestPath(goalNode).getPathlength();
        }
        if (null!=oldBest && null!= newBest && !oldBest.equals(newBest)){
            //System.out.println("!!!! Verbesserung gefunden fr "+this.getName()+" (anderer Nachbarknoten) : "+oldBest.getName()+" -> "+newBest.getName()+" ("+oldDistance+" -> "+") "+newDistance+" fr Ziel : "+goalNode.getName());
            /*if (this.getName().equals("Kln")){
            System.out.println("");
            System.out.println("     Tabelle fr ["+this.getName()+"]");
            Vector probabilityList = getProbabilityList();
            for (int i=0; i<probabilityList.size(); i++){
                RoutingTableEntry entry = (RoutingTableEntry)probabilityList.get(i);
                System.out.println("     "+entry.getNeighbourNode()+" als Weg nach "+entry.getGoalNode()+" Gewichtung "+entry.getPheromoneValue() + " Weglnge : "+entry.getPathlength());
            }
            System.out.println("");
        }*/
        } else
        if (oldDistance!=0.0 && newDistance!=0.0 && oldDistance>newDistance){
            //System.out.println("!!!! Verbesserung gefunden fr "+this.getName()+" (gleicher Nachbarknoten [ "+oldBest.getName()+" ]) : "+oldDistance+" -> "+newDistance+" fr Ziel : "+goalNode.getName());
            /*if (this.getName().equals("Kln")){
            System.out.println("");
            System.out.println("     Tabelle fr ["+this.getName()+"]");
            Vector probabilityList = getProbabilityList();
            for (int i=0; i<probabilityList.size(); i++){
                RoutingTableEntry entry = (RoutingTableEntry)probabilityList.get(i);
                System.out.println("     "+entry.getNeighbourNode()+" als Weg nach "+entry.getGoalNode()+" Gewichtung "+entry.getPheromoneValue() + " Weglnge : "+entry.getPathlength());
            }
            System.out.println("");
            }*/
        }
        /*System.out.println("");
        System.out.println("     Tabelle fr ["+this.getName()+"]");
        Vector probabilityList = getProbabilityList();
        for (int i=0; i<probabilityList.size(); i++){
            RoutingTableEntry entry = (RoutingTableEntry)probabilityList.get(i);
            System.out.println("     "+entry.getNeighbourNode()+" als Weg nach "+entry.getGoalNode()+" Gewichtung "+entry.getPheromoneValue() + " Weglnge : "+entry.getPathlength());
        }
        System.out.println("");*/
    }

    /**
     * Fuegt eine Kante hinzu.
     * @param edge neue Kante.
     */
    public void addEdge(Teilstrecke edge) {
        edges.add(edge);
    }


    public void removeEdge(Teilstrecke edge){
        edges.remove(edge);
    }
    /**
     * Erfragt x-Koordinate.
     * @return x-Koordinate.
     */
    public double getX() {
        return x;
    }
    
    /**
     * Erfragt y-Koordinate.
     * @return y-Koordinate.
     */
    public double getY() {
        return y;
    }
    
    /**
     * Luftlinenentfernung zu anderem Knoten.
     * @param other anderer Knoten.
     * @return Entfernung.
     */
    public double distTo(Node other) {
        double dx = x - other.x;
        double dy = y - other.y;
        return Math.sqrt(dx*dx + dy*dy);
    }
      
    /**
     * Kante an der ein Nachbarknoten sitzt.
     * @param other Nachbarknoten.
     * @return verbindende Kante.
     */
    public Teilstrecke edgeTo(Node other) {
        for (Iterator iter = edges.iterator(); iter.hasNext();) {
            Teilstrecke edge = (Teilstrecke)iter.next();
            if (edge.getPartner(this) == other) {
                return edge;
            }
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return name;
    }
        
    /**
     * Gibt einen Iterator ueber die Nachbarknoten zurueck.
     * @return Iterator ueber Nachbarknoten.
     */
    public Iterator neighbours() {
        return new NodeIterator(this);
    }
    
    private static class NodeIterator implements Iterator {

        private Iterator iter;
        private Node origin;
        
        private NodeIterator(Node origin) {
            this.origin = origin;
            this.iter = origin.edges.iterator();

            //hier wird nach den KFZ-Eigenschaften gefiltert, welche Knoten Nachbarn sind
            Vector filter = new Vector();
            Teilstrecke tempStrecke;
            while (this.iter.hasNext()){
                tempStrecke = (Teilstrecke)this.iter.next();
                if (tempStrecke.istBefahrbarDurchKFZ()){
                    filter.add(tempStrecke);
                }
                //else System.out.println("Eine Teilstrecke wird nicht bercksichtigt, weil sie durch das KFZ nicht befahrbar ist!");
            }
            this.iter = filter.iterator();

        }
        
        /* (non-Javadoc)
         * @see java.util.Iterator#hasNext()
         */
        public boolean hasNext() {
            return iter.hasNext();
        }

        /* (non-Javadoc)
         * @see java.util.Iterator#next()
         */
        public Object next() {
            return ((Teilstrecke)iter.next()).getPartner(origin);
        }

        /* (non-Javadoc)
         * @see java.util.Iterator#remove()
         */
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
