/*
 * Queue.java
 * Created on 23.04.2003
 */
package search.util;

/**
 * Diese Schnittstelle beschreibt eine sequentielle Datenstruktur.
 * Ex gibt zwei grundlegende Operationen zur Vernderung der Struktur, nmlich
 * <code>put</code> um ein neues Datenelement hinzuzufgen und
 * <code>get</code> um ein Datenelement aus der Datenstruktur zu
 * entnehmen. Indizierte oder iterative Zugriffe auf die Daten sind nicht
 * vorgesehen.
 * <p>
 * In den jeweiligen Implementierungen kann die Anordnung der Daten
 * nach unterschiedlichen Kriterien erfolgen. Gedacht ist dabei zum
 * Beispiel an eine FIFO-, LIFO-Struktur oder an eine nach einem
 * Ordnungskriterium geordnete Priorittswarteschlange. 
 * <p>
 * <b>Hinweis:</b> Der Name <code>Queue</code> drckt nur die eingeschrnkte
 * Zugriffsmglichkeit aus, ohne etwas ber die Reihenfolge der Daten
 * auszusagen.
 * 
 * @author Erich Ehses.
 */
public interface Queue {

    /**
     * Fgt einen neuen Wert hinzu.
     * @param neuer Wert.
     */
    public void put(Object p);

    /**
     * Gibt einen Wert zurck und entfernt ihn aus
     * der Queue.
     * @return nchster Wert.
     * @throws java.util.NoSuchElementException wenn kein Wert vorhanden ist.
     */
    public Object get();

    /**
     * berprft, ob die Queue Werte enthaelt.
     * @return <code>true</code> wenn keine Werte vorhanden.
     */
    public boolean isEmpty();
    
    /**
     * Entfernt alle Inhalte der Queue.
     */
    public void clear();
}
