package search.util;
import search.graph.Node;
import search.graph.Path;

import java.util.Vector;

/**
 * In jedem Knoten (Node) befindet sich eine RoutingTable. Die gespeicherten Eintrge (RoutingTableEntry)
 * bieten ein Kriterium anhand dessen entschieden wird welchen Nachbarknoten Ameisen auf
 * dem Weg zu ihrem Ziel besuchen. Fr die Datenpakete ist dies das einzige Kriterium. 
 */

public class RoutingTable {

    public Vector getProbabilityList() {
        return probabilityList;
    }

    public void setProbabilityList(Vector probabilityList) {
        this.probabilityList = probabilityList;
    }

    // enthlt die Wahrscheinlichkeiten einen bestimmten Nachbarknoten als nchsten Schritt zu whlen,
    // wenn man ein bestimmtes Ziel hat
    private Vector probabilityList;

    public RoutingTable(){
         probabilityList = new Vector();
    }


    // findet den Nachbarknoten mit dem hchsten Pheromonwert fr einen bestimmten Zielknoten
    public Node getBestNeighbourNode (Node goalNode){
        int maxPheromoneCount=0;
        Node bestNeighbourNode=null;
        for (int i=0; i<probabilityList.size(); i++){
            RoutingTableEntry entry = (RoutingTableEntry) probabilityList.elementAt(i);
            if (entry.getGoalNode().equals(goalNode)){
                if (entry.getPheromoneValue()> maxPheromoneCount){
                    bestNeighbourNode = entry.getNeighbourNode();
                    maxPheromoneCount = entry.getPheromoneValue();
                }
            }

        }

        return bestNeighbourNode;
    }

    public int getPheromoneValue (Node neighbourNode, Node goalNode){
        int pheromoneValue = 0;
        for (int i=0; i<probabilityList.size(); i++){
            RoutingTableEntry entry = (RoutingTableEntry) probabilityList.elementAt(i);
            if (entry.getNeighbourNode().equals(neighbourNode) && entry.getGoalNode().equals(goalNode)){
                pheromoneValue = entry.getPheromoneValue();    
            }
        }
        return pheromoneValue;
    }

    /*
    erhht den Pheromonwert fr eine Kombination aus Nachbarknoten und Zielknoten wenn eine backwardAnt vorbeikommt
    und der gefundene Weg krzer als der bisherige ist
    */
    public void increasePheromoneValue (Node goalNode, Node lastVisitedNeighbourNode, Path path){

        //System.out.println("increasePheromoneValue mit "+lastVisitedNeighbourNode.getName()+" -> "+goalNode.getName());

        // es muss ein neuer Eintrag angelegt werden da es diese Kombination noch nicht gibt
        boolean create = true;

        // die Lnge des neuen Weges
        double length = path.getLengthFrom(lastVisitedNeighbourNode);

        for (int i=0; i<probabilityList.size(); i++){
             RoutingTableEntry entry = (RoutingTableEntry) probabilityList.elementAt(i);

            //gibt es die Kombination hingegen...
            if (entry.getGoalNode().equals(goalNode) && entry.getNeighbourNode().equals(lastVisitedNeighbourNode)){

                //...muss nur der Pheromonwert erhht...
                //entry.setPheromoneValue(entry.getPheromoneValue()+1);
                //...und nichts mehr angelegt werden
                create = false;

                // Ist es der krzeste Weg?
                /*if (length < getEntryWithShortestPath(goalNode).getPathlength()){
                    //System.out.println("ja, es ist der krzeste Weg ("+length+" km)");
                    entry.setPheromoneValue(entry.getPheromoneValue()+5);
                }*/

                // Ist es ein krzerer Weg als bisher fr diesen Nachbarknoten?
                if (length < entry.getPathlength()){
                    entry.setPathlength(length);
                    //System.out.println("bestehender Eintrag verndert");
                }
            }
        }

        if (create){
            RoutingTableEntry nEntry = new RoutingTableEntry(lastVisitedNeighbourNode,goalNode,0,length);
            // Ist es der erste oder der krzeste Weg?
            if (null == getEntryWithShortestPath(goalNode) || length < getEntryWithShortestPath(goalNode).getPathlength()){
                nEntry.setPheromoneValue(nEntry.getPheromoneValue()+1);
                //System.out.println("create = true -> neuer, krzerer Weg ("+length+" km)");
            }
            probabilityList.add(nEntry);
        }
    }

    // liefert den Eintrag mit der krzesten Weglnge fr ein bestimmtes Ziel
    public RoutingTableEntry getEntryWithShortestPath(Node goalNode){
        RoutingTableEntry entryWithShortestPath = null;
        double minimumLength=0;

        for (int i=0; i<probabilityList.size(); i++){
           RoutingTableEntry entry = (RoutingTableEntry)probabilityList.elementAt(i);

            if (entry.getGoalNode().equals(goalNode)){
                if (i == 0){
                    minimumLength = entry.getPathlength();
                    entryWithShortestPath = entry;
                }
                if (entry.getPathlength() < minimumLength){
                    entryWithShortestPath = entry;
                }
            }
        }

        return entryWithShortestPath;
    }

    public void removeEntries (Node neighbourNode){
        for (int i=0; i<probabilityList.size(); i++){
            RoutingTableEntry entry = (RoutingTableEntry)probabilityList.elementAt(i);
            if (entry.getGoalNode().equals(neighbourNode)){
                probabilityList.remove(entry);
            }
        }
    }
}
