/*
 * File FIFOQueue.java.
 * Created on 10.05.2003
 */
package search.util.example;

import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import search.util.Queue;

/**
 * Diese Realisierung von <code>Queue</code> verfhrt nach dem LIFO-Prinzip.
 * Es handelt sich also um dass, wass man allgemein als Stack
 * bezeichnet.
 * 
 * Diese Implementierung benutzt die Klasse <code>java.util.ArrayList</code>
 * 
 * @author Erich Ehses.
 */
public final class LIFOQueue implements Queue {
    private List data = new ArrayList();

    /* (non-Javadoc)
     * @see util.Queue#put(java.lang.Object)
     */
    public void put(Object p) {
        data.add(p);
    }

    /* (non-Javadoc)
     * @see util.Queue#get()
     */
    public Object get() {
        if (this.isEmpty())
            throw new NoSuchElementException("Queue is empty");
        int last = data.size() - 1;
        Object result = data.get(last);
        data.remove(last);
        return result;
    }

    /* (non-Javadoc)
     * @see util.Queue#isEmpty()
     */
    public boolean isEmpty() {
        return data.isEmpty();
    }

    /* (non-Javadoc)
     * @see util.Queue#clear()
     */
    public void clear() {
        data.clear();
    }

}
