/*
 * File Sort.java.
 * Diese Datei gehrt nicht zur Praktikumsaufgabe. Sie ist ein
 * Beispiel fr den eleganten Sortieralgorithmus
 * "heap sort".
 * Created on 10.05.2003
 */
package search.util.example;

import java.util.Arrays;
import java.util.Comparator;

public class Sort  {
    
    /**
     * Heapsort-Algorithmus, geschrieben fr leichtes Verstndnis.
     * @param a zu sortierendes Feld.
     * @param c anzuwendender Comparator.
     */
    public static void sort(Object[] a, Comparator c) {
        /* 
         * Fr eine optimale Ausfhrung kann man die Anzahl der swap
         * reduzieren. Leider ist dann der Algorithmus nicht mehr ganz
         * so gut lesbar.
         * 
         * Lasse der Reihe nach Feldelemente nach oben "schwimmen".
         * Dabei wird ein Heap aufgebaut, der die Anzahl der Vergleiche
         * minimiert.
         */
        for (int i=1; i<a.length; i++) {
            swim(a, i, c);    
        }
        /*
         * Das grte Element ist am Anfang. Anschlieend wird dieses
         * mit dem letzten Element getauscht und das neue Element
         * am falschen Platz "sinkt" nach unten.
         * Nach jedem Schritt steht das jeweils letzte Element richtig
         * und wird in der Folge nicht mehr angerhrt.
         */
        for (int i = a.length - 1; i>0; i--) {
            swap(a, 0, i);
            sink(a, i, c);
        }
    }
    
    /**
     * Promoviert einen Knoten soweit, bis er auf ein Elter mit
     * besseren Eigenschaften stt.
     * (Der Knoten schwimmt nach oben, bis er seinen Platz gefunden hat.)
     * @param child Index des Kindknotens.
     */
    private static void swim(Object[]a, int child, Comparator c) {
        int parent = (child-1) / 2;
        while (child > 0 && c.compare(a[child], a[parent]) > 0) {
            swap(a, child, parent);
            child = parent;
            parent = (child-1) / 2;
        }
    }
    
    /**
     * Degradiert einen Knoten soweit, bis er auf Kinder mit
     * schlechteren Eigenschaften stt.
     * (Der Knoten sinkt nach unten, bis er seinen Platz gefunden hat.)
     * @param parent Index des Elternknotens.
     */
    private static void sink(Object[] a, int n, Comparator c) {
        int parent = 0, child = 1;
        while (child < n) {
            if (child < n-1 && c.compare(a[child+1], a[child]) > 0)
                child++;
            if (c.compare(a[parent], a[child]) > 0)
                return;
            swap(a, parent, child);
            parent = child;
            child = 2*parent+1;
        }
    }
    
    /**
     * Vertausche zwei Knoten.
     * @param i Erster Knoten.
     * @param j Zweiter Knoten.
     */
    private static void swap(Object[] a, int i, int j) {
        Object temp = a[i];
        a[i] = a[j];
        a[j] = temp;
    }
    
    public static void main(String[] args) {
        String[] a = { "b", "e", "a", "f", "b1", "z", "m", "a0" , "d1", "6"};
        sort(a, new Comparator() {
            public int compare(Object a, Object b) {
                return ((String)a).compareTo(b);
        }});
        System.out.println(Arrays.asList(a)); // so kann man ein Array ausgeben !
    }  
}
